\name{hdpmFitSetup}
\alias{hdpmFitSetup}
\title{Fitted mean functions}
\description{
  Computes posterior expected mean profiles and corresponding
  posterior standard deviations for observed patients}
}
\usage{
hdpmFitSetup(
      n =   100,
      px =  NULL,
      sig = 1,
      nx = 25, xgrid = NULL,
      work.dir=NULL)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{n}{Number of MCMC posterior simulations to be used
    for the fitted profiles.}
  \item{px}{Dimension of covariate vector}
  \item{sig}{Residual standard deviation (dummy)}
  \item{nx}{Size of grid on the time axis}
  \item{xgrid}{Range of grid, as pair of min and max.}
  \item{work.dir}{working directory to save posterior summaries.}
}
\details{
  Evaluates \eqn{E[ m(t; \theta_{ji}) \mid data]}
  (and corresponding posterior standard deviations)
  as a function of time \eqn{t} for all patients in the data sets.
}
\value{
  The function returns a list with elements
  \item{e}{(\code{npatients} by \code{nx}) matrix of
    posterior expected mean functions. Each row corresponds to a
    patient, each column corresponds to a time point in \code{xgrid}.}
  \item{s}{Same for posterior standard deviations.}
  \item{x}{(\code{npatients} by \code{px}) matrix of
    patient-specific covariates}
  \item{xgrid}{grid on the time axis}
}
\references
{
   Mueller, P., Quintana, F. and Rosner, G. (2004).
  ``Hierarchical Meta-Analysis over Related Non-parametric Bayesian
  Models.''
  Journal of the Royal Statistical Society, Series B, 66, 735--749.}
\author{Peter Mueller}
\note{
  The only functions in the source code that are specific
  to the application in Mueller et al. (2004) are the definition of the
  top level sampling model in the file \code{lpost.c}
  and the functions for resampling (transition probabilities in the
  MCMC) for the random effects in the source file \code{hdpm.c}. Both
  source files are in the directory \code{src} of the package
  distribution. Users with experience in coding MCMC could appropriately
  change these functions and recompile the package for a different
  longitudinal mean function.
}
\seealso{
  The functions in this package implement the application in
  \cite{Mueller et al. (2004).}
  Alternatively, the package
  \code{\link[hdpmn:hdpmn-package]{hdpmn}}
  implements the same hierarchical mixture of Dirichlet process models
  for a multivariate normal sampling model and is useful for
  a wider class of applications.
}
\examples{
\dontrun{
require(hdpm)

## data files
data.dir <- system.file("data",package="hdpm")
work.dir <- paste(getwd(),"/tmp/",sep="")
                  
## data files
Y <- file.path(data.dir,"CALGB.txt")
     ## data: responses
Z <- file.path(data.dir,"CALGBz.txt")  
     ## random effects (initial values) and covariates
X <- file.path(data.dir,"CALGBz0.txt")
     ## same as Z, for future patients (for prediction)

## run MCMC -- save working files in work.dir
hdpm(Y=Y,Z=Z,px=3,q=15,cc=15,work.dir=work.dir)

## post-process MCMC output for predictive inference
## save posterior predictive summaries in work.dir
hdpmPredictSetup(X=X,px=3,
                  j=c(0,1,2,1,2),r=c(0,0,0,1,1),
                  xgrid=c(0,21),work.dir=work.dir)

## plot predictive for future patients in study 1 and 2,
for(i in c(5,7)){
  hdpmPredict(j=1,r=0,i=i)
  hdpmPredict(j=2,r=0,i=i)
}

## part of the predictive corresponding to the idiosyncratic measure
for(i in c(5,7)){
  hdpmPredict(j=1,r=1,i=i, work.dir=work.dir)
  hdpmPredict(j=2,r=1,i=i, work.dir=work.dir)
}

## post-process MCMC output for fitted mean curves
fits <- hdpmFitSetup(px=3,xgrid=c(0,21))

## plot fitted mean curves 
par(mfrow=c(2,2),ask=T)
y <- as.matrix(read.table(Y,header=T)) # read in data
exs <- c(10:17,20:27,30:37,90:97)
for(i in exs){
  hdpmFit(fits,i=i)
  idx <- which(y[,1]==i-1)
  points(y[idx,2],y[idx,3],pch=19)
}
}
}